<?php
namespace Teknisa\DF\Modules\Entrada\Controllers;

use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Teknisa\DF\Controllers\TeknisaCrud;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\DF\Modules\Entrada\Services\ImportacaoConsultaNotaFiscalService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Row;
use Zeedhi\Framework\DTO\Request;

class ImportacaoConsultaNotaFiscalController extends TeknisaCrud {
    
    protected $dataSourceName       = 'intgnotafiscal_fornecedor';
    protected $parameterBagColumns  = array("P_CDPAIS", "P_SGESTADO", "P_CDFILIAL", "P_CDFORNECED", "P_DTINICIAL", "P_DTFINAL", "P_DATA", "P_IDSTATUSNFE", "P_CDINSCESTA", "P_NRNOTAFISCINI", "P_NRNOTAFISCFIN", "P_NRNOTA", "P_NRACESSONFE", "P_IDDOCFISCELE");
    
    public function __construct($crudService, ParameterBag $parameterBag, $dfe51000ImportacaoConsultaNotaFiscalService, $importacaoNFSeService, $birtController, $connectionCustom) {
        $this->dfe51000ImportacaoConsultaNotaFiscalService = $dfe51000ImportacaoConsultaNotaFiscalService;
        $this->birtController   = $birtController;
        $this->connectionCustom = $connectionCustom;
        parent::__construct($crudService, $parameterBag);
        $this->importacaoNFSeService = $importacaoNFSeService;
        set_time_limit( 7200 ); // Limite de tempo de execução: 2h. Deixe 0 (zero) para sem limite
        ignore_user_abort( true );
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        // var_dump($request);
        parent::find($request, $response);
        
    }
    
    public function updateNumeroAfXml(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrAcessoNfe = $row["NRACESSONFE"];
        $nrAutoForn  = $row["NRAUTOFORN"];
        $nrSeqItProd = (isset($row["NRSEQITPROD"])) ? $row["NRSEQITPROD"] : '';
        
        $this->dfe51000ImportacaoConsultaNotaFiscalService->updateNumeroAfXml($nrAcessoNfe, $nrAutoForn, $nrSeqItProd);
    }
    
    public function gerarNfXml(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $nrAcessoNfe = $row["NRACESSONFE"];
        $cdTipoOper = $row["CDTIPOOPER"];
        
        $nf = $this->dfe51000ImportacaoConsultaNotaFiscalService->gerarNfXml($nrAcessoNfe, $cdTipoOper);
        
        $response->addDataSet(new DataSet("gerarNfXml", $nf));
    }
    
public function statusNfeXml(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();

        $nrAcessoNfe = $row["NRACESSONFE"];
        $idStatusNfe = $row["IDSTATUSNFE"];
        $cdOperador = $row["CDOPERADOR"];
        $dtHistorico = $row["DTHISTORICO"];
        $idStatus = $row["IDSTATUS"];
        $nrSeqHistor = $this->dfe51000ImportacaoConsultaNotaFiscalService->getSequencialHistoricoXML($nrAcessoNfe);
        $params = array(

            "NRACESSONFE"   =>  $nrAcessoNfe,
            "NRSEQHISTOR"   =>  $nrSeqHistor,
            "DTHISTORICO"   =>  $dtHistorico,
            "IDTIPOOPER"    =>  '2',
            "IDSTATUS"      =>  $idStatus,
            "NRPROTOCOLONFE"=>  ' ',
            "DSOPERACAO"    =>  ' ',
            "CDFILIAL"      =>  ' ',
            "NRLANCTONF"    =>  ' ',
            "CDOPERADOR"    =>  $cdOperador
        );


        if($idStatusNfe == 'N' && $idStatus == 'N' ){
            $mensagem = 'Bloqueada';
            $params["IDSTATUS"] =  'B';
            $idStatusNfe = 'B';
 
        }
        else {
            $mensagem = 'Desbloqueada';
            $params["IDSTATUS"] =  'N';
            $idStatusNfe = 'N';
        } 

        $params["DSOPERACAO"] = $mensagem;

        $this->dfe51000ImportacaoConsultaNotaFiscalService->insereXml($params);
        $this->dfe51000ImportacaoConsultaNotaFiscalService->atualizaStatusXml($nrAcessoNfe, $idStatusNfe);

 
    }
    
    public function getPainelInconsistencias(DTO\Request\Filter $request, DTO\Response $response) {
        $conditions = $request->getFilterCriteria()->getConditions();
        
        foreach($conditions as $condition) {
            if($condition["columnName"] == "NRACESSONFE") $nrAcessoNfe = $condition["value"];
        }
        
        if(!$nrAcessoNfe) $response->addDataSet(new DataSet("consistxml_itpanconsxml", array()));
        else {
            $response->addDataSet(new DataSet("consistxml_itpanconsxml", $this->dfe51000ImportacaoConsultaNotaFiscalService->getPainelInconsistencias($nrAcessoNfe)));
        }
    }
     
    public function getItensInconsistencias(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $request->getFilterCriteria()->getConditions();
        
        foreach($conditions as $condition) {
            if($condition["columnName"] == "NRACESSONFE") $nrAcessoNfe   = $condition["value"];
            else if($condition["columnName"] == "CDPAINEL") $cdPainel = $condition["value"];
        }
        
        if(!$nrAcessoNfe || !$cdPainel) $response->addDataSet(new DataSet("panconsxml_itpanconsxml", array()));
        else {
            $response->addDataSet(new DataSet("panconsxml_itpanconsxml", $this->dfe51000ImportacaoConsultaNotaFiscalService->getItensInconsistencias($nrAcessoNfe, $cdPainel)));
        }
        
    }
    
    public function gerarDanfe(DTO\Request\Filter $request, DTO\Response $response){
        $requestConditions = $request->getFilterCriteria()->getConditions();
        $row = $requestConditions[0]["value"];
        $nracessonfe = $row["NRACESSONFE"];

        try {
            $xml = $this->dfe51000ImportacaoConsultaNotaFiscalService->getXml($nracessonfe);
            $danfe = $this->dfe51000ImportacaoConsultaNotaFiscalService->gerarDanfe($xml, $nracessonfe);
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('danfe', array($danfe) ));

        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function geraRelatorioImpCons(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $rowBirt = new Row();
        
        $rowBirt->set("__report_name", $row["IDREPORT"]);
        $rowBirt->set("P_NRACESSONFE", (isset($row["NRACESSONFE"])) ?  ($row["NRACESSONFE"] === "" ? 'T' : $row["NRACESSONFE"] )  : 'T');
        
        if($row["IDREPORT"] !== 'DFE51002') {
            $rowBirt->set("P_NRACESSONFE", (isset($row["NRACESSONFE"])) ? ($row["NRACESSONFE"] === "" ? 'T' : $row["NRACESSONFE"] )  : 'T');
            $rowBirt->set("P_CDFORNECED",  (isset($row["CDFORNECED"]))  ? ($row["CDFORNECED"] === "" ? 'T' : $row["CDFORNECED"] )   : 'T');
            $rowBirt->set("P_CDSERIE",     (isset($row["CDSERIE"]))     ? ($row["CDSERIE"] === "" ? 'T' : $row["CDSERIE"] )      : 'T');
            $rowBirt->set("P_NRNOTAFISC",  (isset($row["NRNOTAFISC"]))  ? ($row["NRNOTAFISC"] === "" ? 'T' : $row["NRNOTAFISC"] )    : 'T');
        }
        else {
             $rowBirt->set("P_DSINCONSISTENCIA", (isset($row["DSINCONSISTENCIA"])) ?  ($row["DSINCONSISTENCIA"] === "" ? 'T' : $row["DSINCONSISTENCIA"] ): 'T');
        }
        
        if($row["IDREPORT"] != 'DFE51000') {
            $rowBirt->set("P_NMARQUIVO", (isset($row["NMARQUIVO"])) ? ($row["NMARQUIVO"] === "" ? 'T' : $row["NMARQUIVO"] )   : 'T');
            $rowBirt->set("P_DSINCONSISTENCIA", (isset($row["DSINCONSISTENCIA"])) ? ($row["DSINCONSISTENCIA"] === "" ? 'T' : $row["DSINCONSISTENCIA"] ):'T');
        }
        
        $rowBirt->set("P_NRORG", $row["NRORG"] );
        $requestBirt = new Request\Row($rowBirt, "POST", "/createRemoteReport", $request->getUserId(), 'aaa');
        $this->birtController->createRemoteReport($requestBirt, $response);
        
        // $response->addDataSet(new DataSet("panconsxml_itpanconsxml", $this->dfe51000ImportacaoConsultaNotaFiscalFactory->getItensInconsistencias($nrAcessoNfe, $cdPainel)));
    }
    
    public function importarNotasFiscais(DTO\Request\Row $request, DTO\Response $response) {
        $this->dfe51000ImportacaoConsultaNotaFiscalService->limpaLoginXML();
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalService->importarNotas();
        if($retorno["SUCESSOS"] > 0 ) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]));
            $this->dfe51000ImportacaoConsultaNotaFiscalService->callEnviarEmailNfPendente();
        }
        $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();
 
        if(count($erros) > 0) {
             $response->addNotification(new Response\Notification($retorno["ERROS"]));
        }
        foreach ($erros as $erro) {
            $response->addNotification(new Response\Notification($erro));
        }

    }
    
    public function uploadNotasFiscais(DTO\Request\Row $request, DTO\Response $response) {
        $rows = $request->getRow();
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalService->uploadNotas($rows);
        
        
        if($retorno["SUCESSOS"] > 0 ) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]));
        }
        $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();

        if(count($erros) > 0) {
             $response->setError(new Response\Error($retorno["ERROS"], 500));
        }
        foreach ($erros as $erro) {
            $response->addNotification(new Response\Notification($erro));
        }
    }
    
    public function importarFacturacionElectronica(DTO\Request\Row $request, DTO\Response $response) {
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalService->importarFacturacionElectronica();
        if($retorno["SUCESSOS"] > 0 ) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]));
        }
        $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();

        if(count($erros) > 0) {
             $response->addNotification(new Response\Notification($retorno["ERROS"]));
        }
        foreach ($erros as $erro) {
            $response->addNotification(new Response\Notification($erro));
        }

    }
    
    public function importarCTe(DTO\Request $request, DTO\Response $response) {
        $retorno = $this->dfe51000ImportacaoConsultaNotaFiscalService->importarCTe();
        if($retorno["ERROS"] == 0) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]." CTe importados com sucesso."));
        }
        else {
            if($retorno["SUCESSOS"] > 0 ) {
                $response->addMessage(new Response\Message($retorno["SUCESSOS"]." CTe importados com sucesso.<br>".$retorno["ERROS"]." CTe com erro. Verifique as inconsistências."));
            }
            else {
                $response->addMessage(new Response\Message($retorno["ERROS"]." CTe com erro. Verifique as inconsistências."));
            }
            $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();
            foreach ($erros as $erro) {
                $response->addMessage(new Response\Message($erro));
            }
        }
    }
    
    public function getDefaultFilterImportacao(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $defaultFilter = $this->dfe51000ImportacaoConsultaNotaFiscalService->getDefaultFilterImportacao($row);
        
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('defaultFilter', $defaultFilter));
           
    }
    
    public function enviaEmailRelacionamentoPendente(DTO\Request $request, DTO\Response $response) {
        
        $row = $request->getRow();
        
        $emailEnviado  = false;
        
        $entityManager = null;
        
        
        if(isset($row["VPD"]) && isset($row["NRORG"])) {
             $entityManager = $this->connectionCustom->updateVpdConnection($row["NRORG"]);
        }
        
        $itens = $this->dfe51000ImportacaoConsultaNotaFiscalService->getItensPendentesParametrizacao($entityManager);
        
        
        if($itens) {
      
            $htmlPorDestinatario  = $this->dfe51000ImportacaoConsultaNotaFiscalService->getHtmlItensPendentes($itens);
      
            foreach ($htmlPorDestinatario as $destinatario => $item) {
                $emailEnviado = $this->dfe51000ImportacaoConsultaNotaFiscalService->enviarEmailItensPendente($item["HTML"], $destinatario); 
                if($emailEnviado) {
                    $this->dfe51000ImportacaoConsultaNotaFiscalService->updateEmailItemPendente($item["CDFORNECED"], $entityManager);
                }
            }

        }
        
        echo $emailEnviado;
        
    }
     
    public function getFilialCNPJ (DTO\Request\Filter $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceName;
        
        $this->dataSourceName = "filial";
        parent::find($request, $response);
        $rows = $response->getDataSets()[0]->getRows();
        
        foreach($rows as &$filial){
            $this->dfe51000ImportacaoConsultaNotaFiscalService->getFilial($filial['CDEMPRESA'], $filial['NRINSJURFILI']);
        }
        
        $dataSet = new \Zeedhi\Framework\DataSource\DataSet('filial', $rows);
        $response->setDataSets(array($dataSet));
  
    }
    
     public function getInscEstadual(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        $cdfilial = $row['CDFILIAL'];
        $inscestad = $this->dfe51000ImportacaoConsultaNotaFiscalService-> getInscEstadual($cdfilial);
        $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('getInscEstadual', $inscestad));
    }
       
    public function getRepresentante (DTO\Request\Filter $request, DTO\Response $response) {
        $dataSourceName = $this->dataSourceName;
        $this->dataSourceName = "inscrestad";
        parent::find($request, $response);
        $rows = $response->getDataSets()[0]->getRows();
        foreach($rows as &$inscricao){
            if(!$inscricao['NMREPRINSEST'])
                $inscricao['NMREPRINSEST'] = $inscricao['CDINSCESTA'];
        }
        
        $dataSet = new \Zeedhi\Framework\DataSource\DataSet('inscrestad', $rows);
        $response->setDataSets(array($dataSet));
        
    }

    public function importarNFSe(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $retorno = $this->importacaoNFSeService->sincronizarNFSe($row);
        
        if($retorno["SUCESSOS"] > 0 ) {
            $response->addMessage(new Response\Message($retorno["SUCESSOS"]));
            //$this->dfe51000ImportacaoConsultaNotaFiscalService->callEnviarEmailNfPendente();
        }
        $erros = $this->dfe51000ImportacaoConsultaNotaFiscalService->getErros();
 
        if(count($erros) > 0) {
             $response->addNotification(new Response\Notification($retorno["ERROS"]));
        }
        foreach ($erros as $erro) {
            $response->addNotification(new Response\Notification($erro));
        }
    }
    
    
      public function updateDesvinculaAfProduto(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrAcessoNfe = $row["NRACESSONFE"];
        $nrSeqItProd = $row["NRSEQITPROD"];
        $nrAutoForn  = null;
        
        $this->dfe51000ImportacaoConsultaNotaFiscalService-> updateDesvinculaAfProduto($nrAcessoNfe, $nrAutoForn, $nrSeqItProd);
        $this->dfe51000ImportacaoConsultaNotaFiscalService->desvinculaAfProduto($nrAcessoNfe, $nrSeqItProd);

    }
    
    public function updateDesvinculaAfNotafiscal(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrAcessoNfe = $row["NRACESSONFE"];
        $nrAutoForn  =  null;

        $this->dfe51000ImportacaoConsultaNotaFiscalService-> updateDesvinculaAfNotafiscal($nrAcessoNfe, $nrAutoForn);
        $this->dfe51000ImportacaoConsultaNotaFiscalService->desvinculaAfNotafiscal($nrAcessoNfe);
    }
}